import logging
import pkg_resources
import sys

from lxml import etree
from optparse import OptionParser
from six import string_types, integer_types, PY3

if PY3:
    stdout = sys.stdout.buffer
else:
    stdout = sys.stdout

strparam = etree.XSLT.strparam

logger = logging.getLogger('diazo')

namespaces = dict(
    diazo="http://namespaces.plone.org/diazo",
    css="http://namespaces.plone.org/diazo/css",
    old1="http://openplans.org/deliverance",
    old2="http://namespaces.plone.org/xdv",
    oldcss1="http://namespaces.plone.org/xdv+css",
    oldcss2="http://namespaces.plone.org/diazo+css",
    xml="http://www.w3.org/XML/1998/namespace",
    xsl="http://www.w3.org/1999/XSL/Transform",
)


def localname(name):
    return name.rsplit('}', 1)[1]


def namespace(name):
    return name.rsplit('}', 1)[0][1:]


def fullname(namespace, name):
    return '{%s}%s' % (namespace, name)


AC_READ_FILE = etree.XSLTAccessControl(
    read_file=True, write_file=False, create_dir=False, read_network=False,
    write_network=False)
AC_READ_NET = etree.XSLTAccessControl(
    read_file=True, write_file=False, create_dir=False, read_network=True,
    write_network=False)


class CustomResolver(etree.Resolver):
    def __init__(self, data):
        self.data = data

    def resolve(self, url, pubid, context):
        output = self.data.get(url)
        if output is not None:
            return self.resolve_string(output, context)


class LoggingXSLTWrapper:
    def __init__(self, xslt, logger):
        self.xslt = xslt
        self.logger = logger

    def __call__(self, *args, **kw):
        result = self.xslt(*args, **kw)
        for msg in self.xslt.error_log:
            if msg.type == etree.ErrorTypes.ERR_OK:
                self.logger.debug(msg.message)
            else:
                self.logger.debug(msg)
        return result


def pkg_parse(name, parser=None):
    with open(pkg_resources.resource_filename('diazo', name)) as f:
        return etree.parse(f, parser=parser)


def pkg_xsl(name, parser=None):
    return LoggingXSLTWrapper(etree.XSLT(pkg_parse(name, parser)), logger)


def quote_param(value):
    """Quote for passing as an XSL parameter.

    Works with strings, booleans, numbers and None.
    """

    if isinstance(value, string_types):
        return strparam(value)
    elif isinstance(value, bool):
        return value and 'true()' or 'false()'
    elif isinstance(value, integer_types + (float,)):
        value = repr(value)
    elif value is None:
        return '/..'
    else:
        raise ValueError("Cannot convert %s", value)


def split_params(s):
    """Turn foo,bar=baz into {'foo': None, 'bar': 'baz'}
    """

    xsl_params = {}
    for param in s.split(','):
        tokens = [t.strip() for t in param.split('=')]
        xsl_params[tokens[0]] = len(tokens) > 1 and tokens[1] or None
    return xsl_params


def _createOptionParser(usage):
    parser = OptionParser(usage=usage)
    parser.add_option("-o", "--output", metavar="output.xsl",
                      help="Output filename (instead of stdout)",
                      dest="output", default=stdout)
    parser.add_option("-p", "--pretty-print", action="store_true",
                      help="Pretty print output (may alter rendering in "
                           "browser)",
                      dest="pretty_print", default=False)
    parser.add_option("--trace", action="store_true",
                      help="Compiler trace logging",
                      dest="trace", default=False)
    parser.add_option("-a", "--absolute-prefix", metavar="/",
                      help="relative urls in the theme file will be made into "
                           "absolute links with this prefix.",
                      dest="absolute_prefix", default=None)
    parser.add_option("-i", "--includemode", metavar="INC",
                      help="include mode (document, ssi, ssiwait or esi)",
                      dest="includemode", default=None)
    parser.add_option("-n", "--network", action="store_true",
                      help="Allow reads to the network to fetch resources",
                      dest="read_network", default=False)
    parser.add_option("-t", "--theme", metavar="theme.html",
                      help="Theme file",
                      dest="theme", default=None)
    parser.add_option("-r", "--rules", metavar="rules.xml",
                      help="Diazo rules file",
                      dest="rules", default=None)
    parser.add_option("-c", "--custom-parameters",
                      metavar="param1,param2=defaultval",
                      help="Comma-separated list of custom parameter names "
                           "with optional default values that the compiled "
                           "theme will be able accept when run",
                      dest="xsl_params", default=None)
    parser.add_option("-e", "--extra", metavar="extra.xsl",
                      help="Extra XSL to be included in the transform "
                           "(depracated, use inline xsl in the rules instead)",
                      dest="extra", default=None)
    return parser



from __future__ import print_function
#
# Simple test runner for validating different diazo scenarios
#

from lxml import etree
import os
import sys
import difflib
from io import BytesIO, StringIO, open
import unittest
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
import pkg_resources

import diazo.compiler
import diazo.run

from diazo.utils import quote_param
from formencode.doctest_xml_compare import xml_compare
from future.builtins import str


if __name__ == '__main__':
    __file__ = sys.argv[0]

defaultsfn = pkg_resources.resource_filename('diazo.tests',
                                             'default-options.cfg')


class DiazoTestCase(unittest.TestCase):

    writefiles = os.environ.get('DiazoTESTS_WRITE_FILES', False)
    warnings = os.environ.get(
        'DiazoTESTS_WARN', "1").lower() not in ('0', 'false', 'off')

    testdir = None  # override

    @classmethod
    def suiteForParent(cls, parent, prefix):
        """Return a suite of diazo tests, one for each directory in parent.
        """
        suite = unittest.TestSuite()
        for name in os.listdir(parent):
            if name.startswith('.'):
                continue
            path = os.path.join(parent, name)
            if not os.path.isdir(path):
                continue

            contentpath = os.path.join(path, 'content.html')
            if not os.path.isfile(contentpath):
                continue

            test_cls = type('%s-%s' % (prefix, name), (DiazoTestCase,),
                            dict(testdir=path))
            suite.addTest(unittest.makeSuite(test_cls))
        return suite

    def testAll(self):
        self.errors = BytesIO()
        config = configparser.ConfigParser()
        config.read([defaultsfn, os.path.join(self.testdir, "options.cfg")])

        themefn = None
        if config.get('diazotest', 'theme'):
            themefn = os.path.join(self.testdir, config.get('diazotest',
                                                            'theme'))
        contentfn = os.path.join(self.testdir, "content.html")
        rulesfn = os.path.join(self.testdir, "rules.xml")
        xpathsfn = os.path.join(self.testdir, "xpaths.txt")
        xslfn = os.path.join(self.testdir, "compiled.xsl")
        outputfn = os.path.join(self.testdir, "output.html")

        xsl_params = {}
        extra_params = config.get('diazotest', 'extra-params')
        if extra_params:
            for token in extra_params.split(' '):
                token_split = token.split(':')
                xsl_params[token_split[0]] = len(token_split) > 1 and \
                    token_split[1] or None

        if not os.path.exists(rulesfn):
            return

        contentdoc = etree.parse(source=contentfn, base_url=contentfn,
                                 parser=etree.HTMLParser())

        # Make a compiled version
        theme_parser = etree.HTMLParser()
        ct = diazo.compiler.compile_theme(
            rules=rulesfn,
            theme=themefn,
            parser=theme_parser,
            absolute_prefix=config.get('diazotest', 'absolute-prefix'),
            indent=config.getboolean('diazotest', 'pretty-print'),
            xsl_params=xsl_params,
        )

        # Serialize / parse the theme - this can catch problems with escaping.
        cts = etree.tostring(ct, encoding='unicode')
        parser = etree.XMLParser()
        etree.fromstring(cts, parser=parser)

        # Compare to previous version
        if os.path.exists(xslfn):
            with open(xslfn) as f:
                old = f.read()
            new = cts
            if old != new:
                if self.writefiles:
                    with open(xslfn + '.old', 'w') as f:
                        f.write(old)
                if self.warnings:
                    print("WARNING:", "compiled.xsl has CHANGED")
                    for line in difflib.unified_diff(old.split(u'\n'),
                                                     new.split(u'\n'),
                                                     xslfn, 'now'):
                        print(line)

        # Write the compiled xsl out to catch unexpected changes
        if self.writefiles:
            with open(xslfn, 'w') as f:
                f.write(cts)

        # Apply the compiled version, then test against desired output
        theme_parser.resolvers.add(diazo.run.RunResolver(self.testdir))
        processor = etree.XSLT(ct)
        params = {}
        params['path'] = "'%s'" % config.get('diazotest', 'path')

        for key in xsl_params:
            try:
                params[key] = quote_param(config.get('diazotest', key))
            except configparser.NoOptionError:
                pass

        result = processor(contentdoc, **params)

        # Read the whole thing to strip off xhtml namespace.
        # If we had xslt 2.0 then we could use xpath-default-namespace.
        self.themed_string = str(result)
        self.themed_content = etree.ElementTree(
            file=StringIO(self.themed_string), parser=etree.HTMLParser())

        # remove the extra meta content type

        metas = self.themed_content.xpath(
            "/html/head/meta[@http-equiv='Content-Type']")
        if metas:
            meta = metas[0]
            meta.getparent().remove(meta)

        if os.path.exists(xpathsfn):
            with open(xpathsfn) as f:
                for xpath in f.readlines():
                    # Read the XPaths from the file, skipping blank lines and
                    # comments
                    this_xpath = xpath.strip()
                    if not this_xpath or this_xpath[0] == '#':
                        continue
                    assert self.themed_content.xpath(this_xpath), "%s: %s" % (
                        xpathsfn, this_xpath)

        # Compare to previous version
        if os.path.exists(outputfn):
            with open(outputfn) as f:
                old = f.read()
            new = self.themed_string
            if not xml_compare(
                    etree.fromstring(old.strip()),
                    etree.fromstring(new.strip())):
                # if self.writefiles:
                #    open(outputfn + '.old', 'w').write(old)
                for line in difflib.unified_diff(old.split(u'\n'),
                                                 new.split(u'\n'),
                                                 outputfn, 'now'):
                    print(line)
                assert old == new, "output.html has CHANGED"

        # Write out the result to catch unexpected changes
        if self.writefiles:
            with open(outputfn, 'w') as f:
                f.write(self.themed_string)


def test_suite():
    suite = unittest.TestSuite()
    dist = pkg_resources.get_distribution('diazo')
    tests_dir = os.path.join(dist.location, 'diazo', 'tests')
    suite.addTest(DiazoTestCase.suiteForParent(tests_dir, 'Test'))
    if dist.precedence == pkg_resources.DEVELOP_DIST:
        recipes_dir = os.path.join(os.path.dirname(dist.location),
                                   'docs', 'recipes')
        if os.path.exists(os.path.join(recipes_dir, 'diazo-tests-marker.txt')):
            # Could still be a 'System' package.
            suite.addTest(DiazoTestCase.suiteForParent(recipes_dir, 'Recipe'))
    return suite



from setuptools import setup, find_packages
import os.path
import sys

extras_require = {
    'wsgi': ['repoze.xmliter>=0.6', 'WebOb>=1.4'],
    'test': [
        'formencode',
        'repoze.xmliter>=0.6',
        'WebOb>=1.4',
    ]}

if sys.version_info < (2, 7):
    extras_require['test'].append('unittest2')

readme = open("README.rst").read()
changes = open(os.path.join("docs", "CHANGES.txt")).read()
long_desc = readme + '\n\n' + changes

setup(
    name='diazo',
    version='1.2.1',
    description='''Diazo implements a Deliverance like language using a pure
        XSLT engine. With Diazo, you "compile" your theme and ruleset in one
        step, then use a superfast/simple transform on each request thereafter.
        Alternatively, compile your theme during development, check it into
        version control, and not touch Diazo during deployment.''',
    keywords='web theming',
    long_description=long_desc,
    packages=find_packages('lib'),
    package_dir={'': 'lib'},
    include_package_data=True,
    zip_safe=False,
    author='Paul Everitt, Laurence Rowe and Martin Aspeli.',
    author_email='laurence@lrowe.co.uk',
    url="http://diazo.org",
    license='New BSD',
    classifiers=[
        "Development Status :: 6 - Mature",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Topic :: Text Processing :: Markup :: XML",
        ],
    install_requires=[
        'setuptools',
        'lxml',
        'cssselect',
        'future',
        'six'],
    extras_require=extras_require,
    test_suite="diazo.tests.alltests",
    tests_require=extras_require['test'],
    entry_points="""
        [console_scripts]
        diazocompiler = diazo.compiler:main
        diazorun = diazo.run:main
        diazopreprocessor = diazo.rules:main

        [paste.filter_app_factory]
        xslt = diazo.wsgi:XSLTMiddleware
        main = diazo.wsgi:DiazoMiddleware
        """,
)
